package com.tanpu.community.controller;


import com.tanpu.common.api.CommonResp;
import com.tanpu.common.auth.AuthLogin;
import com.tanpu.common.auth.UserHolder;
import com.tanpu.common.exception.BizException;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.req.theme.*;
import com.tanpu.community.api.beans.resp.CreateThemeResp;
import com.tanpu.community.manager.ThemeManager;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
@Slf4j
@RequestMapping(value = "/api/theme")
public class ThemeController {

    @Autowired
    private ThemeManager themeManager;
    @Resource
    private UserHolder userHolder;

    @AuthLogin
    @ApiOperation("发表主题")
    @PostMapping(value = "/publish")
    @ResponseBody
    public CommonResp<CreateThemeResp> publishTheme(@Validated @RequestBody CreateThemeReq req) {
        String userId = userHolder.getUserId();
        return CommonResp.success(themeManager.publishTheme(req, userId));
    }

    @AuthLogin
    @ApiOperation("圈子首页-推荐/关注/热门/最新")
    @PostMapping(value = "/list")
    @ResponseBody
    public CommonResp<List<ThemeQo>> selectInterestList(@Validated @RequestBody ThemeListReq req) {
        String userId = userHolder.getUserId();
        if(StringUtils.isEmpty(userId)){
            throw new BizException("用户为空");
        }
        List<ThemeQo> result = themeManager.queryThemes(req, userId);
        return CommonResp.success(result);
    }

    @AuthLogin
    @ApiOperation("主题正文")
    @GetMapping(value = "/detail")
    @ResponseBody
    public CommonResp<ThemeQo> getThemeMainText(@RequestParam String themeId) {
        String userId = userHolder.getUserId();
        ThemeQo themeQo = themeManager.getDetail(themeId, userId);
        return CommonResp.success(themeQo);
    }

    @AuthLogin
    @ApiOperation("转发主题")
    @PostMapping(value = "/forward")
    @ResponseBody
    public CommonResp<CreateThemeResp> forwardTheme(@Validated @RequestBody ForwardThemeReq forwardThemeReq) {
        String userId = userHolder.getUserId();
        return CommonResp.success(themeManager.forward(forwardThemeReq, userId));
    }

    @AuthLogin
    @ApiOperation("点赞/取消点赞主题")
    @PostMapping(value = "/like")
    @ResponseBody
    public CommonResp<Void> likeOnTheme(@RequestBody LikeThemeReq req) {
        String userId = userHolder.getUserId();
        themeManager.like(req, userId);
        return CommonResp.success();
    }

    @AuthLogin
    @ApiOperation("用户删除自有主题")
    @GetMapping(value = "/delete")
    @ResponseBody
    public CommonResp<Void> delete(@RequestParam(value = "themeId") String themeId) {
        String userId = userHolder.getUserId();
        themeManager.delete(themeId);
        return CommonResp.success();
    }

    @AuthLogin
    @ApiOperation("收藏/取消收藏主题")
    @PostMapping(value = "/collect")
    @ResponseBody
    public CommonResp<Void> bookTheme(@RequestBody CollectThemeReq req) {
        String userId = userHolder.getUserId();
        themeManager.collect(req, userId);
        return CommonResp.success();
    }

    @AuthLogin
    @ApiOperation("举报主题")
    @PostMapping(value = "/report")
    @ResponseBody
    public CommonResp complaintTheme(@RequestBody ReportThemeReq req) {

        return CommonResp.failed("功能暂未开放");
    }

    @AuthLogin
    @ApiOperation("分享主题")
    @GetMapping(value = "/share")
    @ResponseBody
    public CommonResp<Void> shareTheme(String themeId) {
        return CommonResp.failed("功能暂未开放");
    }

    @AuthLogin
    @ApiOperation("屏蔽")
    @GetMapping(value = "/block")
    @ResponseBody
    public CommonResp concealTheme(String themeId) {
        String userId = userHolder.getUserId();
        return CommonResp.success();
    }

    @AuthLogin
    @ApiOperation("关注主题更新数量")
    @GetMapping(value = "/updateCount")
    @ResponseBody
    public CommonResp<Integer> updateCount() {
        String userId = userHolder.getUserId();
        return CommonResp.success(themeManager.getFollowUpdateCount(userId));
    }

    @AuthLogin
    @GetMapping(value = "/test")
    @ResponseBody
    public CommonResp<Integer> test() {
        String userId = userHolder.getUserId();
        return CommonResp.success(themeManager.getFollowUpdateCount(userId));

    }
    }
