package com.tanpu.community.dao.entity.fund;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.util.Date;
import lombok.Builder;
import lombok.Data;

/**
    * 基金基本信息
    */
@ApiModel(value="com-tanpu-community-dao-entity-fund-FundInfo")
@Data
@Builder
@TableName(value = "fund_info")
public class FundInfo implements Serializable {
    /**
     * 基金id
     */
    @TableId(value = "id", type = IdType.INPUT)
    @ApiModelProperty(value="基金id")
    private String id;

    /**
     * 父级产品ID
     */
    @TableField(value = "p_fund_id")
    @ApiModelProperty(value="父级产品ID")
    private String pFundId;

    /**
     * 基金中文全称
     */
    @TableField(value = "fund_name")
    @ApiModelProperty(value="基金中文全称")
    private String fundName;

    /**
     * 基金中文简称
     */
    @TableField(value = "fund_short_name")
    @ApiModelProperty(value="基金中文简称")
    private String fundShortName;

    /**
     * 基金形式：1-公司型，2-合伙型，3-契约型，-1-其他
     */
    @TableField(value = "fund_structure")
    @ApiModelProperty(value="基金形式：1-公司型，2-合伙型，3-契约型，-1-其他")
    private Integer fundStructure;

    /**
     * 私募基金类型：1-信托计划，2-有限合伙，3-券商资管，4-公募专户，5-单账户，6-证券投资基金，7-海外基金，8-期货资管，9-保险资管、10-创业投资基金、11-股权投资基金、12-银行理财、13-类固收信托、-1其他投资基金   		
公募基金类型：1-混合型、2-债券型、3-定开债券、4-联接基金、5-货币型、6-债券指数、7-保本型、8-理财型、9-QDII、10-股票指数、11-QDII-指数、12-股票型、13-固定收益、14-分级杠杆、15-ETF-场内、16-QDII-ETF、17-债券创新-场内、18-封闭式
     */
    @TableField(value = "fund_type")
    @ApiModelProperty(value="私募基金类型：1-信托计划，2-有限合伙，3-券商资管，4-公募专户，5-单账户，6-证券投资基金，7-海外基金，8-期货资管，9-保险资管、10-创业投资基金、11-股权投资基金、12-银行理财、13-类固收信托、-1其他投资基金   		,公募基金类型：1-混合型、2-债券型、3-定开债券、4-联接基金、5-货币型、6-债券指数、7-保本型、8-理财型、9-QDII、10-股票指数、11-QDII-指数、12-股票型、13-固定收益、14-分级杠杆、15-ETF-场内、16-QDII-ETF、17-债券创新-场内、18-封闭式")
    private Integer fundType;

    /**
     * 是否可预约 0:不可预约 1:可预约
     */
    @TableField(value = "`type`")
    @ApiModelProperty(value="是否可预约 0:不可预约 1:可预约")
    private Integer type;

    /**
     * 募集方式：1-私募，2-公募
     */
    @TableField(value = "raise_type")
    @ApiModelProperty(value="募集方式：1-私募，2-公募")
    private Integer raiseType;

    /**
     * 券商资管产品特点
     */
    @TableField(value = "fund_characteristic")
    @ApiModelProperty(value="券商资管产品特点")
    private String fundCharacteristic;

    /**
     * 基础货币,1-人民币，2-港币，3-美元，4-份，-1-其他
     */
    @TableField(value = "base_currency")
    @ApiModelProperty(value="基础货币,1-人民币，2-港币，3-美元，4-份，-1-其他")
    private Integer baseCurrency;

    /**
     * 成立日期
     */
    @TableField(value = "inception_date")
    @ApiModelProperty(value="成立日期")
    private Date inceptionDate;

    /**
     * 注册国家,1-中国大陆、2-香港、3-新加坡、4-开曼群岛、5-台湾、6-英属维尔京群岛BVI、-1-其他
     */
    @TableField(value = "domicile")
    @ApiModelProperty(value="注册国家,1-中国大陆、2-香港、3-新加坡、4-开曼群岛、5-台湾、6-英属维尔京群岛BVI、-1-其他")
    private Integer domicile;

    /**
     * 指数id，以'IN'开头(后加36进制编码格式,不足8位长度左补零） 例：IN00000001
     */
    @TableField(value = "primary_benchmark_id")
    @ApiModelProperty(value="指数id，以'IN'开头(后加36进制编码格式,不足8位长度左补零） 例：IN00000001")
    private String primaryBenchmarkId;

    /**
     * 封闭期，单位：月，-1：不确定，0：无封闭期
     */
    @TableField(value = "lockup_period")
    @ApiModelProperty(value="封闭期，单位：月，-1：不确定，0：无封闭期")
    private Integer lockupPeriod;

    /**
     * 开放日
     */
    @TableField(value = "open_day")
    @ApiModelProperty(value="开放日")
    private String openDay;

    /**
     * 产品存续期，单位：月。-1，不确定，0，无固定期限，999999永续
     */
    @TableField(value = "duration")
    @ApiModelProperty(value="产品存续期，单位：月。-1，不确定，0，无固定期限，999999永续")
    private Integer duration;

    /**
     * 基金投资范围
     */
    @TableField(value = "investment_scope")
    @ApiModelProperty(value="基金投资范围")
    private String investmentScope;

    /**
     * 投资限制
     */
    @TableField(value = "investment_restriction")
    @ApiModelProperty(value="投资限制")
    private String investmentRestriction;

    /**
     * 投资理念
     */
    @TableField(value = "fund_investment_philosophy")
    @ApiModelProperty(value="投资理念")
    private String fundInvestmentPhilosophy;

    /**
     * 投资策略
     */
    @TableField(value = "fund_strategy_description")
    @ApiModelProperty(value="投资策略")
    private String fundStrategyDescription;

    /**
     * 投资顾问id
     */
    @TableField(value = "advisor_id")
    @ApiModelProperty(value="投资顾问id")
    private String advisorId;

    /**
     * 托管银行Id
     */
    @TableField(value = "custodian_id")
    @ApiModelProperty(value="托管银行Id")
    private String custodianId;

    /**
     * 证券经纪人Id
     */
    @TableField(value = "broker_id")
    @ApiModelProperty(value="证券经纪人Id")
    private String brokerId;

    /**
     * 期货经纪人id
     */
    @TableField(value = "broker_future_id")
    @ApiModelProperty(value="期货经纪人id")
    private String brokerFutureId;

    /**
     * 外包机构Id
     */
    @TableField(value = "liquidation_agency_id")
    @ApiModelProperty(value="外包机构Id")
    private String liquidationAgencyId;

    /**
     * 基金管理公司Id
     */
    @TableField(value = "trust_id")
    @ApiModelProperty(value="基金管理公司Id")
    private String trustId;

    /**
     * 投资顾问Id
     */
    @TableField(value = "investment_consultant_id")
    @ApiModelProperty(value="投资顾问Id")
    private String investmentConsultantId;

    /**
     * 行政管理人Id
     */
    @TableField(value = "administrator_id")
    @ApiModelProperty(value="行政管理人Id")
    private String administratorId;

    /**
     * 法律顾问Id
     */
    @TableField(value = "legal_counsel_id")
    @ApiModelProperty(value="法律顾问Id")
    private String legalCounselId;

    /**
     * 审计机构
     */
    @TableField(value = "auditor_id")
    @ApiModelProperty(value="审计机构")
    private String auditorId;

    /**
     * 净值披露频率 1:天 2：周 3：月 4：半月 5：季度
     */
    @TableField(value = "nav_frequency")
    @ApiModelProperty(value="净值披露频率 1:天 2：周 3：月 4：半月 5：季度")
    private Integer navFrequency;

    /**
     * 产品业绩披露标识：1-AAA，2-AA，3-A，4-A-，5-A+
     */
    @TableField(value = "performance_disclosure_mark")
    @ApiModelProperty(value="产品业绩披露标识：1-AAA，2-AA，3-A，4-A-，5-A+")
    private Integer performanceDisclosureMark;

    /**
     * 备案编码
     */
    @TableField(value = "register_number")
    @ApiModelProperty(value="备案编码")
    private String registerNumber;

    /**
     * 备案日期
     */
    @TableField(value = "register_date")
    @ApiModelProperty(value="备案日期")
    private Date registerDate;

    /**
     * 基金在前台是否可见:1-可见 0-不可见
     */
    @TableField(value = "isvisible")
    @ApiModelProperty(value="基金在前台是否可见:1-可见 0-不可见")
    private Integer isvisible;

    /**
     * 是否分级：1-分级，0-不分级;
     */
    @TableField(value = "istiered")
    @ApiModelProperty(value="是否分级：1-分级，0-不分级;")
    private Integer istiered;

    /**
     * 是否参与排名，1-参与排名 0-不参与排名 	
     */
    @TableField(value = "is_ranking")
    @ApiModelProperty(value="是否参与排名，1-参与排名 0-不参与排名 	")
    private Integer isRanking;

    /**
     * 是否参与评级，1-参与评级 0-不参与评级	
     */
    @TableField(value = "is_rating")
    @ApiModelProperty(value="是否参与评级，1-参与评级 0-不参与评级	")
    private Integer isRating;

    /**
     * 基金协会特别提示
     */
    @TableField(value = "special_tips")
    @ApiModelProperty(value="基金协会特别提示")
    private String specialTips;

    /**
     * 基金协会链接
     */
    @TableField(value = "amac_url")
    @ApiModelProperty(value="基金协会链接")
    private String amacUrl;

    /**
     * 净值来源说明：1-托管外包、2-信托券商官网、3-私募机构
     */
    @TableField(value = "nav_source_type")
    @ApiModelProperty(value="净值来源说明：1-托管外包、2-信托券商官网、3-私募机构")
    private Integer navSourceType;

    /**
     * 组合目标
     */
    @TableField(value = "combine_target")
    @ApiModelProperty(value="组合目标")
    private String combineTarget;

    /**
     * 适合群体
     */
    @TableField(value = "fit_group")
    @ApiModelProperty(value="适合群体")
    private String fitGroup;

    /**
     * 组合点评
     */
    @TableField(value = "combine_comment")
    @ApiModelProperty(value="组合点评")
    private String combineComment;

    /**
     * 调仓点评
     */
    @TableField(value = "transfer_comment")
    @ApiModelProperty(value="调仓点评")
    private String transferComment;

    /**
     * 基金描述
     */
    @TableField(value = "desc_info")
    @ApiModelProperty(value="基金描述")
    private String descInfo;

    /**
     * 融智策略分类，1-股票策略，2-宏观策略，3-管理期货，4-事件驱动，5-相对价值策略，6-固定收益策略，7-组合基金，8-复合策略，-1-其它策略
     */
    @TableField(value = "strategy")
    @ApiModelProperty(value="融智策略分类，1-股票策略，2-宏观策略，3-管理期货，4-事件驱动，5-相对价值策略，6-固定收益策略，7-组合基金，8-复合策略，-1-其它策略")
    private Integer strategy;

    /**
     * 融智子策略分类:1010-主观多头 1020-股票多空 1030-量化多头 2010-宏观策略 3010-主观趋势 3020-主观套利 3030-量化趋势 3040-量化套利 3050-管理期货复合 4010-并购重组 4020-定向增发 4030-大宗交易 4040-事件驱动复合 5010-股票市场中性 5020-套利 5030-相对价值复合 6010-纯债策略 6020-强债策略 6030-固收复合 7010-MOM 7020-FOF 8010-主观多策略 8020-量化多策略 -1-其他策略'
     */
    @TableField(value = "substrategy")
    @ApiModelProperty(value="融智子策略分类:1010-主观多头 1020-股票多空 1030-量化多头 2010-宏观策略 3010-主观趋势 3020-主观套利 3030-量化趋势 3040-量化套利 3050-管理期货复合 4010-并购重组 4020-定向增发 4030-大宗交易 4040-事件驱动复合 5010-股票市场中性 5020-套利 5030-相对价值复合 6010-纯债策略 6020-强债策略 6030-固收复合 7010-MOM 7020-FOF 8010-主观多策略 8020-量化多策略 -1-其他策略'")
    private Integer substrategy;

    @TableField(value = "has_nav")
    @ApiModelProperty(value="")
    private Integer hasNav;

    /**
     * 风险等级
     */
    @TableField(value = "risk_level")
    @ApiModelProperty(value="风险等级")
    private Integer riskLevel;

    @TableField(value = "create_by")
    @ApiModelProperty(value="")
    private String createBy;

    @TableField(value = "create_time")
    @ApiModelProperty(value="")
    private Date createTime;

    @TableField(value = "update_by")
    @ApiModelProperty(value="")
    private String updateBy;

    @TableField(value = "update_time")
    @ApiModelProperty(value="")
    private Date updateTime;

    /**
     * 数据来源 1:tanpu，2:私募排排
     */
    @TableField(value = "data_sources")
    @ApiModelProperty(value="数据来源 1:tanpu，2:私募排排")
    private Integer dataSources;

    /**
     * 0:待上架  1：已上架 2：已下架
     */
    @TableField(value = "`status`")
    @ApiModelProperty(value="0:待上架  1：已上架 2：已下架")
    private Integer status;

    /**
     * 删除标识 0:否 1:是
     */
    @TableField(value = "delete_tag")
    @ApiModelProperty(value="删除标识 0:否 1:是")
    private Integer deleteTag;

    /**
     * 排序
     */
    @TableField(value = "sort")
    @ApiModelProperty(value="排序")
    private Integer sort;

    @TableField(value = "apply_day")
    @ApiModelProperty(value="")
    private String applyDay;

    @TableField(value = "redeem_day")
    @ApiModelProperty(value="")
    private String redeemDay;

    /**
     * 机构ID
     */
    @TableField(value = "org_id")
    @ApiModelProperty(value="机构ID")
    private String orgId;

    private static final long serialVersionUID = 1L;

    public static final String COL_ID = "id";

    public static final String COL_P_FUND_ID = "p_fund_id";

    public static final String COL_FUND_NAME = "fund_name";

    public static final String COL_FUND_SHORT_NAME = "fund_short_name";

    public static final String COL_FUND_STRUCTURE = "fund_structure";

    public static final String COL_FUND_TYPE = "fund_type";

    public static final String COL_TYPE = "type";

    public static final String COL_RAISE_TYPE = "raise_type";

    public static final String COL_FUND_CHARACTERISTIC = "fund_characteristic";

    public static final String COL_BASE_CURRENCY = "base_currency";

    public static final String COL_INCEPTION_DATE = "inception_date";

    public static final String COL_DOMICILE = "domicile";

    public static final String COL_PRIMARY_BENCHMARK_ID = "primary_benchmark_id";

    public static final String COL_LOCKUP_PERIOD = "lockup_period";

    public static final String COL_OPEN_DAY = "open_day";

    public static final String COL_DURATION = "duration";

    public static final String COL_INVESTMENT_SCOPE = "investment_scope";

    public static final String COL_INVESTMENT_RESTRICTION = "investment_restriction";

    public static final String COL_FUND_INVESTMENT_PHILOSOPHY = "fund_investment_philosophy";

    public static final String COL_FUND_STRATEGY_DESCRIPTION = "fund_strategy_description";

    public static final String COL_ADVISOR_ID = "advisor_id";

    public static final String COL_CUSTODIAN_ID = "custodian_id";

    public static final String COL_BROKER_ID = "broker_id";

    public static final String COL_BROKER_FUTURE_ID = "broker_future_id";

    public static final String COL_LIQUIDATION_AGENCY_ID = "liquidation_agency_id";

    public static final String COL_TRUST_ID = "trust_id";

    public static final String COL_INVESTMENT_CONSULTANT_ID = "investment_consultant_id";

    public static final String COL_ADMINISTRATOR_ID = "administrator_id";

    public static final String COL_LEGAL_COUNSEL_ID = "legal_counsel_id";

    public static final String COL_AUDITOR_ID = "auditor_id";

    public static final String COL_NAV_FREQUENCY = "nav_frequency";

    public static final String COL_PERFORMANCE_DISCLOSURE_MARK = "performance_disclosure_mark";

    public static final String COL_REGISTER_NUMBER = "register_number";

    public static final String COL_REGISTER_DATE = "register_date";

    public static final String COL_ISVISIBLE = "isvisible";

    public static final String COL_ISTIERED = "istiered";

    public static final String COL_IS_RANKING = "is_ranking";

    public static final String COL_IS_RATING = "is_rating";

    public static final String COL_SPECIAL_TIPS = "special_tips";

    public static final String COL_AMAC_URL = "amac_url";

    public static final String COL_NAV_SOURCE_TYPE = "nav_source_type";

    public static final String COL_COMBINE_TARGET = "combine_target";

    public static final String COL_FIT_GROUP = "fit_group";

    public static final String COL_COMBINE_COMMENT = "combine_comment";

    public static final String COL_TRANSFER_COMMENT = "transfer_comment";

    public static final String COL_DESC_INFO = "desc_info";

    public static final String COL_STRATEGY = "strategy";

    public static final String COL_SUBSTRATEGY = "substrategy";

    public static final String COL_HAS_NAV = "has_nav";

    public static final String COL_RISK_LEVEL = "risk_level";

    public static final String COL_CREATE_BY = "create_by";

    public static final String COL_CREATE_TIME = "create_time";

    public static final String COL_UPDATE_BY = "update_by";

    public static final String COL_UPDATE_TIME = "update_time";

    public static final String COL_DATA_SOURCES = "data_sources";

    public static final String COL_STATUS = "status";

    public static final String COL_DELETE_TAG = "delete_tag";

    public static final String COL_SORT = "sort";

    public static final String COL_APPLY_DAY = "apply_day";

    public static final String COL_REDEEM_DAY = "redeem_day";

    public static final String COL_ORG_ID = "org_id";
}